/* Copyright (c) 2011 Asustor Inc. All rights reserved. */
/**
 * @class AS.ARC.apps.exfat.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.exfat.core.applyInfo', {
    extend: 'Ext.container.Container',

    alias: ['widget.asExfatApplyStatus'],

    layout: 'hbox',

    width: '100%',

    style: 'opacity:0;',

    cls: 'exfat-applyInfo',

    imgUrl: 'resources/images/icons/',

    fadeOutId: '',

    applyChanged: function () {
        var me = this,
            ctIems = me.items;

        ctIems.get('icon').setSrc(me.imgUrl + 'ok.png');
        ctIems.get('message').setText(_S('COMMON', 'APPLY_CHANGE'));
        ctIems.get('message').el.dom.setStyle('font-size: 12px; color: #008000;');
        me.doDisplay();
    },

    applyNotChanged: function () {
        var me = this,
            ctIems = me.items;

        ctIems.get('icon').setSrc(me.imgUrl + 'warning2.png');
        ctIems.get('message').setText(_S('COMMON', 'SETTING_NO_CHANGE'));
        ctIems.get('message').el.dom.setStyle('font-size: 12px; color: #cc6633;');

        me.doDisplay();
    },

    resetDisplay: function () {
        var me = this;
        clearTimeout(me.fadeOutId);
        me.el.stopAnimation();
        me.el.dom.style.opacity = 0;
    },

    doDisplay: function () {
        var me = this;
        clearTimeout(me.fadeOutId);
        me.el.stopAnimation();
        me.el.fadeIn();
        me.fadeOutId = Ext.defer(function () {
            if (me.el) {
                me.el.fadeOut({duration: 3000});
            }
        }, 5000);
    },

    initComponent: function () {
        var me = this;
       
        Ext.apply(me, {
            items: [{
                xtype: 'image',
                itemId: 'icon',
                width: 16,   
                height: 16,
                src: me.imgUrl + 'warning2.png'
            }, {
                xtype: 'label',
                itemId: 'message',
                text: _S('COMMON', 'SETTING_NO_CHANGE'),
                cls: 'exfat-apply-status-text'
            }]
        });
        me.callParent(arguments);
    },

    beforeDestroy: function () {
        var me = this;
        me.callParent(arguments);
    }
});

Ext.define('AS.ARC.apps.exfat.core', {
	extend: 'Ext.util.Observable',
        
    constructor: function (config) {
		Ext.apply(this, config);
		
		this.callParent();	
        this.initConfig();
		this.init();
	},

    initConfig: function(){
        var fn = this;

        // APP resource path
        fn.apiUrl = AS.ARC.util.getUserAppsPath() + 'exfat/' + 'exfat.cgi';
        fn.imageUrl = AS.ARC.util.getUserAppsPath() + 'exfat/' + 'images/';	
        // ADM resource path
        fn.imgPath =  '/portal/resources/images/icons/';

        // CGI parameters config
        fn.cgiConfig = {
            status:  {
                action: {act: 'status'}
            },
            register: {
                action: {act: 'register'},
                params: {auth_code: ''} 
                
            },
            unregister: {
                action: {act: 'unregister'},
                params: {auth_code: ''}
            }
        };

        fn.cgiErrMsg =  {
            5000: _S('ERROR', 'SESSION_TIMEOUT'),
            5601: _S('EXFAT', 'NET_ERR'),
            5008: _S('EXFAT', 'KEY_ALREADY_USE')
        };

        // JSON after calling CGI
        fn.licenseCodeStatus =  {};
        
        fn.images =  {
            ok: fn.imgPath + AS.ARC.util.fixDc('ok.png'),
            fail: fn.imgPath + AS.ARC.util.fixDc('error2.png'),
            loading: fn.imgPath + AS.ARC.util.fixDc('loading.gif'),
            side: fn.imageUrl + AS.ARC.util.fixDc('side.png')
        };

        fn.asLicenseStoreUrl = {
            chinese: 'http://seller.pcstore.com.tw/S122080002/',
            english: 'https://shop.asustor.com/'
        };

        // exFAT license code input max length
        fn.tfCodeMaxLength =  4;
        fn.tfCodeWidth =  70;
        fn.placeholder = '----';
        
        // exFAT app, license code
        var exFatLicenseRE = {
            exFatLicenseCode: /^((?:(?![01IOio])[0-9A-ZAa-z])+)$/
        };
        Ext.form.VTypes.exFatLicenseCodeMask = /((?:(?![01IOio])[0-9A-ZAa-z])+)/;
        Ext.form.VTypes.exFatLicenseCodeRe = exFatLicenseRE.exFatLicenseCode;
        Ext.form.VTypes.exFatLicenseCode = function (v) {
            return Ext.form.VTypes.exFatLicenseCodeRe.test(v);
        };
        Ext.form.VTypes.exFatLicenseCodeText = Ext.String.format(_S('MESSAGE', 'HF00027') , "1, 0, I, O");
    },

    codeInputControler: function(){
        var fn = this;

        var getInputCompleteCode =  function(){
            var result = '';

            result = fn.win.down('#tfCode1').getValue() + '-' + 
                fn.win.down('#tfCode2').getValue() + '-' +   
                fn.win.down('#tfCode3').getValue() + '-' +
                fn.win.down('#tfCode4').getValue();
                
            return result;   
        };

        var setAllDisable = function(){
            fn.win.down('#tfCode1').disable();
            fn.win.down('#tfCode2').disable();
            fn.win.down('#tfCode3').disable();
            fn.win.down('#tfCode4').disable();
            return true;
        };

        var setAllEmpty = function(){
            fn.win.down('#tfCode1').setValue('');
            fn.win.down('#tfCode2').setValue('');
            fn.win.down('#tfCode3').setValue('');
            fn.win.down('#tfCode4').setValue('');
            return true;
        };

        var setAllBlur = function(){
            fn.win.down('#tfCode1').blur();
            fn.win.down('#tfCode2').blur();
            fn.win.down('#tfCode3').blur();
            fn.win.down('#tfCode4').blur();
            return true;
        };

        var setBtnActionDisable = function(){
            fn.win.down('#btnAction').disable();
            return true;
        };

        var setBtnActionEnable = function(){
            fn.win.down('#btnAction').enable();
            return true;
        };

        var isAllCorrect = function(){
            var result = true;

            var setBindBtnStatus = function(){
                if (result) {
                    fn.win.down('#btnAction').enable();
                } else if (!result) {
                    fn.win.down('#btnAction').disable();
                }
            };

            fn.win.down('#registerCodeCt').items.each(function(item, index){
                if (item.xtype === "textfield") {
                    if (item.getValue().length !== fn.tfCodeMaxLength || item.isValid() === false) {
                        result = false;
                    }
                }
            });

            setBindBtnStatus();
            return result;
        };

        return {
            getInputCompleteCode: getInputCompleteCode,
            setAllDisable: setAllDisable,
            setAllEmpty: setAllEmpty,
            setAllBlur: setAllBlur,
            setBtnActionDisable: setBtnActionDisable,
            setBtnActionEnable:setBtnActionEnable,
            isAllCorrect: isAllCorrect
        };
    },

    handleEnter: function(field, e, app){
        var fn = app;

        var isAllFieldFillInCompletely = function(){
            var result = true;
            field.up().items.each(function(item, index){
                if (item.xtype === "textfield") {
                    item.blur();
                    if (item.getValue().length !== field.maxLength) {
                        result = false;
                    }
                }
            });

            return result;
        };
        
        if (e.getKey() == e.ENTER) {
            if(isAllFieldFillInCompletely()){
                fn.win.down('#btnAction').fireEvent('click');
            }
        }
    },

    handleRegister: function() {
        var fn = this;

        fn.win.el.mask(_S('COMMON', 'APPLYING'));
        fn.cgiConfig.register.params.auth_code = fn.codeInputControler().getInputCompleteCode();

        var doUpdate = function(){
            fn.handleStatus(function(){
                var params = {
                    callFrom: 'register'
                }

                fn.callbackByHandleStatus(params);
            });
        };

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, fn.cgiConfig.register.action),
            method: 'GET',
            params: fn.cgiConfig.register.params,
            success: function (json, opts) {

                doUpdate();
            },
            failure: function (json, opts) {

                if (json.error_code === 5601) {
                    AS.ARC.util.showMsgWindow(fn.cgiErrMsg, json, fn.win);
                } else if (json.error_code === 5008)  {
                    fn.win.down('#fieldRegisterFail').setValue(fn.cgiErrMsg[5008]);
                    doUpdate();
                } else {
                    // other error -> _S('EXFAT', 'LICENSE_FAIL')
                    fn.win.down('#fieldRegisterFail').setValue(_S('EXFAT', 'LICENSE_FAIL'));
                    doUpdate();
                }
            }
        });
    },

    handleUnregister: function() {
        var fn = this;

        fn.win.el.mask(_S('COMMON', 'APPLYING'));
        fn.cgiConfig.unregister.params.auth_code = fn.codeInputControler().getInputCompleteCode();

        var doUpdate = function(){
            fn.handleStatus(function(){
                fn.callbackByHandleStatus({
                    callFrom: 'unRegister'
                });
            });
        };

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, fn.cgiConfig.unregister.action),
            method: 'GET',
            params: fn.cgiConfig.unregister.params,
            success: function (json, opts) {

                doUpdate();
            },
            failure: function (json, opts) {
                if (json.error_code === 5601) {
                    AS.ARC.util.showMsgWindow(fn.cgiErrMsg, json, fn.win);
                } else {
                    doUpdate();
                }
            }
        });
    },

    callbackByHandleStatus: function(action){
        var fn = this,
            formPanel = fn.win.down('[itemId=formPanel]'),
            asExfatApplyStatus = formPanel.down('[itemId=asExfatApplyStatus]');

        var handleRegister = function(){
            if (!fn.licenseCodeStatus.register) {
                fn.win.down('#fieldRegisterFail').show();
                fn.win.down('#fieldAlreadyRegister').hide();
            } else if (fn.licenseCodeStatus.register) {
                fn.win.down('#fieldRegisterFail').hide();
                fn.codeInputControler().setAllEmpty();
                fn.codeInputControler().setAllBlur();
            }
        };

        var handleUnregister = function(){
            if (fn.licenseCodeStatus.register) {
                fn.win.down('#fieldRegisterFail').show();
                fn.win.down('#fieldAlreadyRegister').hide();
            } else if (!fn.licenseCodeStatus.register) {
                fn.win.down('#fieldRegisterFail').hide();
                fn.codeInputControler().setAllEmpty();
                fn.codeInputControler().setAllBlur();
            }
        };

        var handleStatusInfoTable = {
            'register': handleRegister,
            'unRegister': handleUnregister
        }

        handleStatusInfoTable[action.callFrom]();
        fn.win.el.unmask();
    },

    handleStatus: function(cbForReigsterOrUnregister) {
        var fn = this,
            originalStatus = {},
            formPanel = fn.win.down('[itemId=formPanel]'),
            asExfatApplyStatus = formPanel.down('[itemId=asExfatApplyStatus]');

        fn.win.el.mask(_S('COMMON', 'LOADING'));

        var setStatus = function(){
            if (fn.licenseCodeStatus.register === false) {
                fn.win.down('#btnAction').setText(_S('EXFAT', 'BIND'));                
                fn.win.down('#tfCode1').focus(true, 10);
                
            } else if (fn.licenseCodeStatus.register) {
                fn.win.down('#btnAction').setText(_S('EXFAT', 'UNBIND'));
                fn.win.down('#fieldAlreadyRegister').show();
            }

            fn.win.el.unmask();
        };
        
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fn.apiUrl, fn.cgiConfig.status.action),
            method: 'GET',
            success: function (json, opts) {
                originalStatus = fn.licenseCodeStatus;
                fn.licenseCodeStatus = json;

                fn.win.down('#fieldAlreadyRegister').hide();
                fn.win.down('#btnAction').disable();
                setStatus();

                if (Ext.isFunction(cbForReigsterOrUnregister)) {
                    cbForReigsterOrUnregister();
                    if (originalStatus.register !== fn.licenseCodeStatus.register) {
                        asExfatApplyStatus.applyChanged();
                    } else {
                        asExfatApplyStatus.applyNotChanged();
                    }
				}
            },
            failure: function (json, opts) {
                AS.ARC.util.showMsgWindow(fn.cgiErrMsg, json, fn.win);
            }
        });
    },  

    addPasteEvent: function(field){
        var fn = this;
        field.el.dom.querySelector('input[type=text]').addEventListener('paste', function(e){
            var clipboardData, 
                pastedData;

            // Stop data actually being pasted into div
            e.stopPropagation();
            e.preventDefault();

            // Get pasted data via clipboard API
            clipboardData = e.clipboardData || window.clipboardData;
            pastedData = clipboardData.getData('Text');

            if (fn.codeInputControler().isAllCorrect() === false) {
                fn.win.down('#btnAction').disable();
            }

            var splitPastedData = pastedData.split('-'),
                code1 = splitPastedData[0],
                code2 = splitPastedData[1]
                code3 = splitPastedData[2]
                code4 = splitPastedData[3];


            fn.win.down('#tfCode1').setValue(code1);
            fn.win.down('#tfCode2').setValue(code2);
            fn.win.down('#tfCode3').setValue(code3);
            fn.win.down('#tfCode4').setValue(code4);
        });
    },

    getMainPanel: function(){
        var fn = this,
            exfatHyperLink = '';

        if (AS.ARC.user.lang === 'zh-TW') {
            exfatHyperLink = fn.asLicenseStoreUrl.chinese;
        } else {
            exfatHyperLink = fn.asLicenseStoreUrl.english;
        }
        var fieldStep1 = Ext.create(Ext.form.field.Display, {
            itemId: 'fieldStep1',
            fieldLabel: Ext.String.format(_S('EXFAT', 'STEP'), 1),
            cls: 'fexfat-desc displayField',
            value: Ext.String.format(_S('EXFAT', 'STEP_1_DESC'), '<a class="hyperlink-string" target="_blank" href="' + exfatHyperLink + '">' + _S('EXFAT', 'STEP_1_DESC_HYPERLINK') + '</a>'),
        });

        var fieldStep2 = Ext.create(Ext.form.field.Display, {
            itemId: 'fieldStep2',
            fieldLabel: Ext.String.format(_S('EXFAT', 'STEP'), 2),
            cls: 'fexfat-desc displayField',
            value: _S('EXFAT', 'STEP_2_DESC')
        });

        var fieldRegisterFail = Ext.create(Ext.form.field.Display, {
            itemId: 'fieldRegisterFail',
            hidden: true,
            value: _S('EXFAT', 'LICENSE_FAIL'),
            cls: 'fexfat-desc displayField'
        });

        var fieldAlreadyRegister = Ext.create(Ext.form.field.Display, {
            itemId: 'fieldAlreadyRegister',
            hidden: true,
            value: _S('EXFAT', 'LICENSE_ALREADY_ENABLED'),
            cls: 'fexfat-desc displayField'
        });

        var fieldNote = Ext.create(Ext.form.field.Display, {
            itemId: 'fieldNote',
            value: _S('COMMON', 'NOTE') + ': ' + _S('EXFAT', 'MAINTAIN_INTERNET_CONN'),
            cls: 'note-string displayField'
        });

        var tfCode1 = Ext.create(Ext.form.field.Text, {
            itemId: 'tfCode1',
            width: fn.tfCodeWidth,
            maxLength: fn.tfCodeMaxLength,   
            enforceMaxLength: true,
            emptyText: fn.placeholder, // in html is "placeholder" attribute
            cls: 'exfat-textfield',
            vtype: 'exFatLicenseCode',
            listeners: {
                specialkey: function(field, e){
                    fn.handleEnter(field, e, fn);
                },
                focus: function(field, eOpts){
                    
                },
                change: function(field, newValue, oldValue, eOpts){
                    field.setValue(field.getValue().toUpperCase());
                    if (newValue.length === fn.tfCodeMaxLength) {
                        if (fn.codeInputControler().isAllCorrect()) {
                            fn.codeInputControler().setBtnActionEnable();
                        } else {
                            tfCode2.enable();
                            tfCode2.focus(true, 10);
                        }
                    } else {
                        fn.codeInputControler().setBtnActionDisable();
                    }
                },
                afterrender: function(field){
                    fn.addPasteEvent(field);
                }
            }
        });	

        var tfCode2 = Ext.create(Ext.form.field.Text, {
            itemId: 'tfCode2',
            width: fn.tfCodeWidth,
            maxLength: fn.tfCodeMaxLength,
            enforceMaxLength: true,
            //disabled: true,
            emptyText: fn.placeholder, 
            cls: 'exfat-textfield',
            vtype: 'exFatLicenseCode',
            listeners: {
                specialkey: function(field, e){
                    fn.handleEnter(field, e, fn);
                },
                change: function(field, newValue, oldValue, eOpts){
                    if (!field.isValid()) return;
                    field.setValue(field.getValue().toUpperCase());
                    if (newValue.length === fn.tfCodeMaxLength) {
                        if (fn.codeInputControler().isAllCorrect()) {
                            fn.codeInputControler().setBtnActionEnable();
                        } else {
                            tfCode3.enable();
                            tfCode3.focus(true, 10);
                        }
                    } else {
                        fn.codeInputControler().setBtnActionDisable();
                    }
                }
            }
        });	

        var tfCode3 = Ext.create(Ext.form.field.Text, {
            itemId: 'tfCode3',
            width: fn.tfCodeWidth,
            maxLength: fn.tfCodeMaxLength,
            enforceMaxLength: true,
            //disabled: true,
            emptyText: fn.placeholder, 
            cls: 'exfat-textfield',
            vtype: 'exFatLicenseCode',
            listeners: {
                specialkey: function(field, e){
                    fn.handleEnter(field, e, fn);
                },
                change: function(field, newValue, oldValue, eOpts){
                    if (!field.isValid()) return;
                    field.setValue(field.getValue().toUpperCase());
                    if (newValue.length === fn.tfCodeMaxLength) {
                        if (fn.codeInputControler().isAllCorrect()) {
                            fn.codeInputControler().setBtnActionEnable();
                        } else {
                            tfCode4.enable();
                            tfCode4.focus(true, 10);
                        }
                    } else {
                        fn.codeInputControler().setBtnActionDisable();
                    }
                }
            }
        });		

        var tfCode4 = Ext.create(Ext.form.field.Text, {  
            itemId: 'tfCode4',
            width: fn.tfCodeWidth,
            maxLength: fn.tfCodeMaxLength,
            enforceMaxLength: true,
            //disabled: true,
            emptyText: fn.placeholder, 
            cls: 'exfat-textfield',
            vtype: 'exFatLicenseCode',
            listeners: {
                specialkey: function(field, e){
                    fn.handleEnter(field, e, fn);
                },
                change: function(field, newValue, oldValue, eOpts){
                    if (!field.isValid()) return;
                    field.setValue(field.getValue().toUpperCase());
                    if (newValue.length === fn.tfCodeMaxLength) {
                        if (fn.codeInputControler().isAllCorrect()) {
                            fn.codeInputControler().setBtnActionEnable();
                        }
                    } else {
                        fn.codeInputControler().setBtnActionDisable();
                    }
                }
            }
        });	

        fn.btnAction = Ext.create(Ext.button.Button, {  
            itemId: 'btnAction',
            text: AS.ARC.datas.defaultValue,
            disabled: true,
            listeners: {
                click: function(btn){
                    if (fn.licenseCodeStatus.register) {
                        fn.handleUnregister();
                    } else if (!fn.licenseCodeStatus.register){
                        fn.handleRegister();
                    }
                    
                }
            }
        });		

        var imgLicenseCodeStatus = Ext.create('Ext.Img', {
            itemId: 'imgLicenseCodeStatus',
            src: '',
            hidden: true,
            cls: 'img-license-code-status'
        });

        var registerCodeCt = Ext.create(Ext.container.Container, {
            itemId: 'registerCodeCt',
            cls: 'as-flexbox exfat-register-ct',
            items: [
                tfCode1,
                {
                    xtype: 'component',
                    cls: 'exfat-code-field-hyphen',
                    html: ' - '
                },
                tfCode2,
                {
                    xtype: 'component',
                    cls: 'exfat-code-field-hyphen',
                    html: ' - '
                },
                tfCode3,
                {
                    xtype: 'component',
                    cls: 'exfat-code-field-hyphen',
                    html: ' - '
                },
                tfCode4,
                {
                    xtype: 'component',
                    width: 5,
                    html: '  '
                },
                imgLicenseCodeStatus
            ]
        });		

        var mainPanel = Ext.create('AS.ARC.formBase', {
            itemId: 'formPanel',
            height: '100%',
			page: fn,
            border: false,
			items: [
                {
                    xtype: 'asFlexbox',
                    height: '100%',
                    items: [{
                        xtype: 'container',
                        cls: 'exfat-side'
                    }, {
                        xtype: 'panel',
                        border: false,
                        width: '100%',
                        height: '100%',
                        items:[{
                            xtype: 'container',
                            cls: 'exfat-content common-layout-vertical-3-0',
                            items: [
                                fieldStep1,
                                fieldStep2,
                                registerCodeCt,
                                fieldRegisterFail,
                                fieldAlreadyRegister,
                                fieldNote
                            ]
                        }],
                        dockedItems: [{
                            xtype: 'toolbar',
                            dock: 'bottom',
                            ui: 'footer',
                            height: 60,
                            cls: 'exfat-footer',
                            items: [
                                {
                                    xtype: 'asExfatApplyStatus',
                                    itemId: 'asExfatApplyStatus'
                                }, {
                                    xtype:'tbfill',
                                },
                                fn.btnAction
                            ]
                        }],
                    }]
                }
            ],
			loadData: function () {
				
			}
		});

        return mainPanel;
    },

    init: function () {
		var fn = this;
        
		this.win = fn.desktop.createWindow({
			app: fn.app,
			itemId: 'app-exfat',
			id: fn.id,
			resizable: true,
			maximizable: true,
			border: false,
			items: [
                fn.getMainPanel()
            ],
			listeners: {
                afterrender: function(win){
                    win.header.addCls('as-window-header-no-tab');

                    fn.handleStatus();
                }
            }
		});
	}
});

/**
 * @class AS.ARC.apps.takeasy.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.exfat.main', {
	extend: 'AS.ARC._appBase',

	appTag: 'exfat',
	title: _S('EXFAT', 'EXFAT'),
	appMaxNum: 1,
	appOpenNum: 0,
	appIsReady: true,
	appWins: [],
	
	createWindow: function () {
		var desktop = this.core.getDesktop(),			
			app = this;
		
		if ((this.appOpenNum === this.appMaxNum) || !this.appIsReady) {
			this.appWins[0].show();
			return;
		}
		
		// Set appIsReady to false
		this.appIsReady = false;

		var exfat = new AS.ARC.apps.exfat.core({
			app: this,
			desktop: desktop,
			id: this.id
		});
		
		exfat.win.on('render', function () {
			app.appOpenNum++;
			app.appIsReady = true;
		});		
	
		exfat.win.on('beforeclose', function () {
			app.appOpenNum--;
			app.appIsReady = true;
			app.appWins.pop();
		});		
		exfat.win.show();
		
		// Get all wins for handling
		this.appWins.push(exfat.win);
	}
});